"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var ts = require("typescript");
var print_1 = require("./print");
/**
 * Checks if the node is exported.
 */
function isNodeExported(node) {
    // Parse the modifier array for the export keyword. If it is found
    // and the node.parent is a sourcefile, return true
    // This only returns top level exports
    var modifiers = node.modifiers;
    if (modifiers) {
        for (var i = 0; i < modifiers.length; i++) {
            if (modifiers[i].kind === ts.SyntaxKind.ExportKeyword) {
                return node.parent.kind === ts.SyntaxKind.SourceFile;
            }
        }
    }
    return false;
}
function getType(prop) {
    var unionType = prop.type;
    if (unionType && unionType.types) {
        return {
            type: 'string',
            values: unionType.types.map(function (i) { return i.getText(); }),
        };
    }
    //noinspection TypeScriptUnresolvedFunction
    return {
        type: prop.type.getText(),
    };
}
function getMethods(checker, type, classDeclaratinNode) {
    return classDeclaratinNode.members
        .map(function (i) { return ({ name: i.name.getText() }); });
}
function getProperties(checker, type, interfaceDeclaratinNode) {
    return type.getProperties()
        .filter(function (i) { return i.valueDeclaration.parent === interfaceDeclaratinNode; })
        .map(function (i) {
        var symbol = checker.getSymbolAtLocation(i.valueDeclaration.name);
        var prop = i.valueDeclaration;
        var typeInfo = getType(prop);
        return {
            name: i.getName(),
            text: i.valueDeclaration.getText(),
            type: typeInfo.type,
            values: typeInfo.values,
            isRequired: !prop.questionToken,
            comment: print_1.syntaxKindToName(i.valueDeclaration.kind) // ts.displayPartsToString(symbol.getDocumentationComment()).trim(),
        };
    });
}
var SourceFile = (function () {
    function SourceFile(sourceFile, checker) {
        this.checker = checker;
        this.variables = [];
        this.interfaces = [];
        this.classes = [];
        var nodes = [];
        this.findAllNodes(sourceFile, nodes);
        this.allNodes = nodes;
        this.variables = this.allNodes
            .filter(function (i) { return i.kind === ts.SyntaxKind.VariableStatement; })
            .map(function (i) { return i; })
            .filter(function (i) { return i.declarationList.declarations
            && i.declarationList.declarations.length === 1
            && i.declarationList.declarations[0].name.kind === ts.SyntaxKind.Identifier; })
            .map(function (i) {
            var d = i.declarationList.declarations[0];
            var identifier = d.name;
            var type = checker.getTypeAtLocation(identifier);
            //d.name.
            return {
                name: identifier.text,
                exported: isNodeExported(i),
                type: type.symbol.getName(),
            };
        });
        this.interfaces = this.allNodes
            .filter(function (i) { return i.kind === ts.SyntaxKind.InterfaceDeclaration; })
            .map(function (i) { return i; })
            .map(function (i) {
            var symbol = checker.getSymbolAtLocation(i.name);
            var type = checker.getTypeAtLocation(i.name);
            console.log("########### " + type.getProperties().length);
            return {
                name: symbol.name,
                comment: ts.displayPartsToString(symbol.getDocumentationComment()).trim(),
                exported: isNodeExported(i),
                properties: getProperties(checker, type, i),
            };
        });
        this.classes = this.allNodes
            .filter(function (i) { return i.kind === ts.SyntaxKind.ClassDeclaration; })
            .map(function (i) { return i; })
            .map(function (i) {
            var symbol = checker.getSymbolAtLocation(i.name);
            var type = checker.getTypeAtLocation(i.name);
            return {
                name: symbol.name,
                exported: isNodeExported(i),
                baseTypes: type.getBaseTypes().map(function (i) { return i.symbol.name; }),
                methods: getMethods(checker, type, i),
            };
        });
        /*
        TODO: create methods that will grab the following:
        - all interfaces (type, exported)
        - all variables (type, exported)
        - all classes (type, exported)

        - check exported variables + classes if they are look like react components
            - React.Component, React.PureComponent, React.StatelessComponent
            - () => any - how to find this out? one input parameter which is type of interface?
        */
    }
    SourceFile.prototype.findAllNodes = function (rootNode, result) {
        var _this = this;
        result.push(rootNode);
        ts.forEachChild(rootNode, function (node) {
            _this.findAllNodes(node, result);
        });
    };
    return SourceFile;
}());
exports.SourceFile = SourceFile;
//# sourceMappingURL=sourceFile.js.map